% AUTHOR: Filippo Piccinini (f.piccinini@unibo.it)
% DATE: 15 January 2020
% NAME: demo01
% 
% This script allows to test the Jaccard Index function, to estimate the
% Jaccard Index according to the metric described in the Kaggle Data
% Science Bowl competition 2018:
% https://www.kaggle.com/c/data-science-bowl-2018/overview/evaluation
%
% The inputs of this function are two 3D matrices contining objects
% labelled with a single ID value.

% Clean the Matlab Workspace 
clc, clear, close all

% Path of the segmentations to be compared
GroundTruth_path = 'TestDataset01\Neurosphere\Spheroid_GroundTruth.tif';
SegmentationTool1_path = 'TestDataset01\Neurosphere\Spheroid_Segmentation1.tif';

% Multipage images converted to matrices
GroundTruth_stack = read_stack(GroundTruth_path);
SegmentationTool1_stack = read_stack(SegmentationTool1_path);

% Jaccard Index single objects
thLevelMin = 0.50;
thLevelStep = 0.02;
thLevelMax = 0.55;
[metricKaggleDataScienceBowl2018value, JI_singleObjects] = metricKaggleDataScienceBowl2018(GroundTruth_stack, SegmentationTool1_stack, thLevelMin, thLevelStep, thLevelMax);

% Final Jaccard Index value
JIvalue = mean(JI_singleObjects);
JIstd = std(JI_singleObjects);

% To obtain the number of ground truth objects for which at least an overlapping object was found in the other mask
GT_Unique = unique(GroundTruth_stack(:));
ST_Unique = unique(SegmentationTool1_stack(:));
GT_UniqueWo0 = GT_Unique; GT_UniqueWo0(GT_UniqueWo0==0)=[];
ST_UniqueWo0 = ST_Unique; ST_UniqueWo0(ST_UniqueWo0==0)=[];
NoObjGT = length(GT_UniqueWo0);
NoObjST = length(ST_UniqueWo0);
gtObjPair = length(find(JI_singleObjects~=0));

% Output message
message = ['The ground truth mask contains: ' num2str(NoObjGT) ' objects. The compared mask contains: ' num2str(NoObjST) '.'];
disp(message)
message = ['The ground truth mask contains: ' num2str(NoObjGT) ' objects and ' num2str(gtObjPair) ' objects have at least one overlapping object in the compared mask.'];
disp(message)
message = ['The global Jaccard Index value (i.e. mean of the single ' num2str(NoObjGT) ' Jaccard Index values) is: ' num2str(JIvalue) ', with a standard deviation of: ' num2str(JIstd) '.'];
disp(message)