function mask3DDilated = dilate_mask3D(mask3DOriginal, radiousSphere)
% AUTHOR: Filippo Piccinini (f.piccinini@unibo.it)
% DATE: 29 May 2019
% NAME: dilate_mask3D
% 
% This function allows to dilate a mask containing more objects identified
% by single ID values
%
% USAGE: 
% mask3DDilated = dilate_mask3D(mask3D, 2);
%
% INPUT PARAMETERS:
% 	mask3DOriginal  3D mask with each object labelled with a
%                   different ID value.
% 	radiousSphere   radious of the sphere to be considered for dilation.
%
% OUTPUT:
%   mask3DDilated   3D mask dilated.
%
% See also imdilate

% BIOMAG Toolbox
% Copyright  2019 Peter Horvath and Filippo Piccinini, 
% Hungarian Academy of Sciences, Biological Research Centre (BRC),
% Szeged, Hungary. All rights reserved.
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

    if radiousSphere<1
        err_dlg = errordlg('radiousSphere must be a positive number!');
        %waitfor(err_dlg);
        return
    end
    
    se = strel('sphere',radiousSphere);
    
    uni = unique(mask3DOriginal);
 
    mask3DDilated = zeros(size(mask3DOriginal));
    for lab=1:numel(uni)
        if(uni(lab)>0)
            mask_lab = mask3DOriginal==uni(lab);
            mask_lab_dil = imdilate(mask_lab, se);
            mask3DDilated(mask_lab_dil) = uni(lab);
        end
    end
    
    flag_showResults = 0;
    view_slice = 1;
    if flag_showResults == 1
        
        subplot(1,2,1);
        imagesc(mask3DOriginal(:, :, view_slice));
        axis image;

        subplot(1,2,2);
        imagesc(mask3DDilated(:, :, view_slice));
        axis image;
    end
    
end