function write_stack(fname, im)
% AUTHOR: Filippo Piccinini
% DATE: 29 May 2019
% NAME: write_stack
% 
% This function allows to write a multi-page 8-bit image starting from a 
% 3D matrix. The original matrix is converted into 8-bit format.
%
% USAGE: 
% write_stack('C:\Desktop\MultipageFile_001.tif', Matrix3D);
%
% INPUT PARAMETERS:
% 	fname           Path and name of the image to be saved.
% 	im              3D matrix to be saved as multi-page image
%
% See also imwrite

% BIOMAG Toolbox
% Copyright  2019 Peter Horvath and Filippo Piccinini, 
% Hungarian Academy of Sciences, Biological Research Centre (BRC),
% Szeged, Hungary. All rights reserved.
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

    %Conversion to 8-bit values
    if isa(im,'double')
        im  = uint8(im);
    elseif isa(im, 'uint16')
        im  = 255.*double(im)./(2^16-1); 
    elseif isa(im, 'uint32')
        im  = 255.*double(im)./(2^32-1); 
    elseif isa(im, 'uint64')
        im  = 255.*double(im)./(2^64-1); 
    end

    %Delete file if existing
    warning off
    delete(fname)
    warning on
    
    %Write file
    for i=1:size(im,3)
        imwrite(im(:,:,i), fname, 'writemode', 'append');
    end
    
end